import type { LanguageRegistration, ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, StyleSheet, Text, type TextStyle, View, type ViewStyle } from 'react-native'

export interface SyntaxHighlighterProps {
  text: string
  language: string | LanguageRegistration
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  debugMode?: boolean
}

let highlighterInstance: Awaited<ReturnType<typeof createHighlighterCore>> | null = null
const loadedLanguages = new Set<string>()

async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
      themes: [],
      langs: [],
    })
  }
  return highlighterInstance
}

function normalizeLanguage(language: string | LanguageRegistration): LanguageRegistration {
  if (typeof language === 'string') {
    return {
      name: language,
      scopeName: `source.${language}`,
      aliases: [language],
      embeddedLangs: [],
      patterns: [],
      repository: {},
    }
  }

  // If it's an array (which seems to be happening in your case), take the first item
  if (Array.isArray(language)) {
    const firstLang = language[0]
    if (typeof firstLang === 'string') {
      return normalizeLanguage(firstLang)
    }
    return {
      ...firstLang,
      name: firstLang.name || firstLang.id || 'unknown',
      scopeName: firstLang.scopeName || `source.${firstLang.name || firstLang.id || 'unknown'}`,
      aliases: firstLang.aliases || [],
      embeddedLangs: firstLang.embeddedLangs || [],
      patterns: firstLang.patterns || [],
      repository: firstLang.repository || {},
    }
  }

  // If it's an object but not properly formatted
  if (typeof language === 'object') {
    return {
      ...language,
      name: language.name || 'unknown',
      scopeName: language.scopeName || `source.${language.name || 'unknown'}`,
      aliases: language.aliases || [],
      embeddedLangs: language.embeddedLangs || [],
      patterns: language.patterns || [],
      repository: language.repository || {},
    }
  }

  // Fallback
  return {
    name: 'plaintext',
    scopeName: 'source.plaintext',
    aliases: ['plaintext', 'text'],
    embeddedLangs: [],
    patterns: [],
    repository: {},
  }
}

async function ensureLanguageLoaded(highlighter: Awaited<ReturnType<typeof createHighlighterCore>>, language: string | LanguageRegistration) {
  const normalizedLang = normalizeLanguage(language)
  const langId = normalizedLang.name

  if (!loadedLanguages.has(langId)) {
    await highlighter.loadLanguage(normalizedLang)
    loadedLanguages.add(langId)
  }

  return langId
}

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ text, language, theme, fontSize = 14, textStyle, scrollViewProps, containerStyle, debugMode = false }) => {
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)

  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true)
        const highlighter = await getHighlighter()

        if (debugMode) {
          // eslint-disable-next-line no-console
          console.log('Highlighter initialization state:', {
            loadedLanguages: Array.from(loadedLanguages),
            inputText: text.substring(0, 100) + (text.length > 100 ? '...' : ''),
            languageType: typeof language === 'string' ? 'string' : 'LanguageRegistration',
            language: normalizeLanguage(language),
          })
        }

        const langId = await ensureLanguageLoaded(highlighter, language)

        // Ensure theme has a name property
        const themeName = typeof theme === 'string' ? theme : theme.name
        if (!themeName) {
          throw new Error('Theme must have a name property')
        }

        await highlighter.loadTheme(theme)

        // Highlight with explanations enabled
        const result = highlighter.codeToTokens(text, {
          lang: langId,
          theme: themeName,
          includeExplanation: true,
        })

        if (debugMode && result.tokens[0]) {
          // eslint-disable-next-line no-console
          console.log(
            'First line token analysis:',
            result.tokens[0].map(token => ({
              content: token.content,
              color: token.color,
              scopes: token.explanation?.[0]?.scopes.map(s => s.scopeName),
            })),
          )
        }

        setTokens(result.tokens)
        setError(null)
      }
      catch (err) {
        if (debugMode) {
          console.error('Highlighting error:', {
            error: err,
            message: err instanceof Error ? err.message : 'Unknown error',
            state: {
              text: text.substring(0, 100) + (text.length > 100 ? '...' : ''),
              language: normalizeLanguage(language),
              loadedLanguages: Array.from(loadedLanguages),
            },
          })
        }
        setError(err instanceof Error ? err.message : 'Failed to highlight code')
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme, debugMode])

  const renderToken = (token: ThemedToken, index: number) => {
    let color = token.color
    if (!color && token.explanation?.[0]?.scopes) {
      for (const scope of token.explanation[0].scopes) {
        if (scope.themeMatches?.[0]?.settings.foreground) {
          color = scope.themeMatches[0].settings.foreground
          break
        }
      }
    }

    const defaultColor = theme.colors?.['editor.foreground'] || '#e1e4e8'

    const tokenStyle: TextStyle = {
      fontSize,
      color: color || defaultColor,
      ...(token.fontStyle === 1 && { fontStyle: 'italic' as const }),
      ...(token.fontStyle === 2 && { fontWeight: 'bold' as const }),
      ...(token.fontStyle === 4 && { textDecorationLine: 'underline' as const }),
    }

    const style = StyleSheet.flatten([textStyle, tokenStyle])

    return (
      <Text key={`${token.offset}-${index}`} style={style}>
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => (
    <View key={index} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
      {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
    </View>
  )

  if (loading) {
    return (
      <View style={[{ backgroundColor: theme.colors?.['editor.background'], padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ backgroundColor: theme.colors?.['editor.background'], padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor: theme.colors?.['editor.background'] }, scrollViewProps?.style]} contentContainerStyle={[containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
